/*
 *  videoMirror.cpp
 *  openFrameworks
 *
 *  Created by Robert Carlsen on 29.01.2009.
 *  Copyright 2009 recv'd productions. All rights reserved.
 *
 */

#include "videoMirror.h"

VideoMirror::VideoMirror() {
    init(20,20,320,240);
}

VideoMirror::VideoMirror(int _x,int _y) {
    init(_x,_y,320,240);
}

VideoMirror::VideoMirror(int _x,int _y, int _w, int _h) {
    init(_x,_y,_w,_h);
}

void VideoMirror::init(int _x, int _y, int _w, int _h){
    x = _x;
    y = _y;
    camWidth 		= _w;	// try to grab at this size. 
	camHeight 		= _h;
	
	vidGrabber.setVerbose(true);
	vidGrabber.initGrabber(camWidth,camHeight);
	
	videoInverted 	= new unsigned char[camWidth*camHeight*3];
	videoMirrored 	= new unsigned char[camWidth*camHeight*3];
	videoTexture.allocate(camWidth,camHeight, GL_RGB);
    
    selectedPixel = new unsigned char[3];
}

VideoMirror::~VideoMirror(){
    
}

void VideoMirror::update() {
    vidGrabber.grabFrame();
	
	if (vidGrabber.isFrameNew()){
		int totalPixels = camWidth*camHeight*3;
		unsigned char * pixels = vidGrabber.getPixels();
		for (int i = 0; i < totalPixels; i++){
            // this is for mirroring the texture:
			// the array is :rgb[1]rgb[2]rgb[3]rgb[4]
            // we want      :rgb[4]rgb[3]rgb[2]rgb[1]
            // without the mod %3 below we'd get:   bgr[4]bgr[3]bgr[2]bgr[1]
            
            int loc = i%3; 
            videoMirrored[(i/(camWidth*3))*(camWidth*3) - (i-loc+3)%(camWidth*3) + loc] = pixels[i]; // this does all the pixel shifting
            //videoInverted[(i/(camWidth*3))*(camWidth*3) - i%(camWidth*3)] = 255 - pixels[i];
            
            // really neat image artifacting. mouseOffset is just a variable int
            // videoMirrored[(i/(camWidth*3))*(camWidth*3) - (i-mouseOffset-loc)%(camWidth*3)] = pixels[i];
            
		}
		videoTexture.loadData(videoMirrored, camWidth,camHeight, GL_RGB);
	}
}

void VideoMirror::draw(){
    /*
    int margin = 20;
    int largeW = (ofGetScreenWidth()-margin*2)/2;
    int largeH = int((camHeight/float(camWidth))*largeW);
    videoTexture.draw(margin,margin,largeW,largeH);
    */
    videoTexture.draw(x,y,camWidth,camHeight);
}

unsigned char * VideoMirror::getPixel(int _x, int _y){
    // given a coordinate, retrieve the color values from the pixel array
    int loc = ((_x-x) + ((_y-y) * camWidth)) * 3;
     
    //check that we're within bounds
    bool inBounds = 1;
    if(_x<x || _x > x+camWidth || _y<y || _y > y+camHeight || loc > (camWidth*camHeight*3)) inBounds = 0;
    
    for(int i=0;i<3;i++){
        if(inBounds)
            selectedPixel[i] = videoMirrored[loc+i];
        else
            selectedPixel[i] = 0;

    }
    
    return selectedPixel;
}












